//************************************************
//*  ImageIO Interface DLL 32 Bits functions     *
//*  Library Version 7.1                         *
//*  Copyright 2019  All rigths reserved         *
//*  RECOGNIFORM TECHNOLGIES SPA                 *
//*  www.recogniform.com - info@recogniform.com  *
//************************************************
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace ImagingDemo
{
    public static class RecoIO
    {
        internal const string recoIO = "recoio.dll";

	//  Goal: nitialize the engine
	//  Requires: company name, license string
	//  Result: session handle
        [DllImport(recoIO, EntryPoint = "IO_Init")]
        internal static extern int IO_Init(string Name, string Key);

	//  Goal: deinitialize the engine
	//  Requires: session handle
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_Done")]
        internal static extern void IO_Done(int SessionHandle);

	//  Goal: load an image from a TIF file
	//  Requires: session handle, file name, page number
	//  Result: DIB handle
        [DllImport(recoIO, EntryPoint = "IO_LoadTIFImage")]
        internal static extern int IO_LoadTIFImage(int SessionHandle, string FileName, int PageNumber);

	//  Goal: save a DIB as a TIFF file with custom compression type
	//  Requires: session handle, DIB handle, file name, compression type
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_SaveTIFImage")]
        internal static extern void IO_SaveTIFImage(int SessionHandle, int DIBHandle, string FileName, int Compression);

	//  Goal: retrieve the number of pages stored in a TIF file
	//  Requires: session handle, file name
	//  Result: number of pages found
        [DllImport(recoIO, EntryPoint = "IO_CountTIFImages")]
        internal static extern int IO_CountTIFImages(int SessionHandle, string FileName);

	//  Goal: load an image from a BMP file
	//  Requires: session handle, filename
	//  Result: DIB handle
        [DllImport(recoIO, EntryPoint = "LoadBMPImage")]
        internal static extern int LoadBMPImage(int SessionHandle, string FileName);

	//  Goal: save a DIB as a BMP file
	//  Requires: session handle, DIB handle, file name
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_SaveBMPImage")]
        internal static extern void IO_SaveBMPImage(int SessionHandle, int DIBHandle, string FileName);

	//  Goal: load an image from a JPG file
	//  Requires: session handle, file name
	//  Result: DIB handle
        [DllImport(recoIO, EntryPoint = "IO_LoadJPGImage")]
        internal static extern int IO_LoadJPGImage(int SessionHandle, string FileName);

	//  Goal: save a DIB as a JPG file
	//  Requires: session handle, DIB handle, file name, Jpegquality
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_SaveJPEGImage")]
        internal static extern void IO_SaveJPEGImage(int SessionHandle, int DIBHandle, string FileName, int Quality);

	//  Goal: load an image from a PDF file
	//  Requires: session handle, file name, page number, DPI Rendering
	//  Result: DIB handle
        [DllImport(recoIO, EntryPoint = "IO_LoadPDFImage")]
        internal static extern int IO_LoadPDFImage(int SessionHandle, string FileName, int PageNumber, int DPIRendering);

	//  Goal: save a DIB as a PDF file with default compression and custom JPEG quality
	//  Requires: session handle, DIB handle, file name, jpeg quality (if used)
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_SavePDFImage")]
        internal static extern void IO_SavePDFImage(int SessionHandle, int DIBHandle, string FileName, int JPEGQuality);

	//  Goal: retrieve the number of images stored in a PDF file
	//  Requires: session handle, file name
	//  Result: number of images found
        [DllImport(recoIO, EntryPoint = "IO_CountPDFImages")]
        internal static extern int IO_CountPDFImages(int SessionHandle, string FileName);

	//  Goal: free the memory associate to and image
	//  Requires: session handle, DIB handle
	//  Result: n/a
		[DllImport(recoIO, EntryPoint = "IO_FreeImage")]
        internal static extern void IO_FreeImage(int SessionHandle, int DIBHandle);

	//  Goal: retrieve information from an image
	//  Requires: session handle, DIB handle, width(out), height(out), xres(out), yres(out), bits (out)
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_GetImageInfo")]
        internal static extern int IO_GetImageInfo(int SessionHandle, int DIBHandle, ref int Width, ref int Height, ref int XRes, ref int YRes, ref int Bits);

	//  Goal: duplicate the image handle
	//  Requires: session handle, DIB handle
	//  Result: DIB handle of new image
        [DllImport(recoIO, EntryPoint = "IO_Duplicate")]
        internal static extern int IO_Duplicate(int SessionHandle, long DIBHandle);

	//  Goal: create a subimage from an image
	//  Requires: session handle, DIB handle, left, top, right and bottom output long buffer
	//  Result: DIB handle of new image
        [DllImport(recoIO, EntryPoint = "IO_GetSubImage")]
        internal static extern int IO_GetSubImage(int SessionHandle, int DIBHandle, int left, int top, int right, int bottom);

	//  Goal: create a resized gray image from a monocrome image
	//  Requires: session handle, DIB handle, zoom factor (1..100)
	//  Result: DIB handle of new image
        [DllImport(recoIO, EntryPoint = "IO_GetScaleToGrayImage")]
        internal static extern int IO_GetScaleToGrayImage(int SessionHandle, int DIBHandle, int ZoomFactor);

	//  Goal: show an image in a draw context
	//  Requires: session handle, DIB handle, DC handle, x and y position
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_ShowImage")]
        internal static extern void IO_ShowImage(int SessionHandle, int DIBHandle, int DC, int X, int Y);

	//  Goal: show an image in a draw context
	//  Requires: session handle, DIB handle, DC handle, x and y position, w and h size
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_ShowStretchedImage")]
        internal static extern void IO_ShowStretchedImage(int SessionHandle, int DIBHandle, int DC, int X, int Y, int W, int H);

	//  Goal: show dialog allowing scanner selection
	//  Requires: session handle, window handle
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_SelectScanner")]
        internal static extern void IO_SelectScanner(int SessionHandle, int WinHandle);

	//  Goal: show scanning dialog allowing user to set parameters and scan
	//  Requires: session handle, window handle
	//  Result: n/a        
        [DllImport(recoIO, EntryPoint = "IO_SetScannerCallback")]
        internal static extern void IO_SetScannerCallback(int SessionHandle, Delegate CallbackProc);

	//  Goal: show scanning dialog allowing user to set parameters and scan
	//  Requires: session handle, window handle
	//  Result: DIB handle of last scanned image
        [DllImport(recoIO, EntryPoint = "IO_ScanImage")]
        internal static extern int IO_ScanImage(int SessionHandle, int WinHandle);

	//  Goal: start scanning without show scanning dialog
	//  Requires: session handle,bit per pixel,resolution,contrast,brightness,width,height, adf,duplex
	//  Result: DIB handle of last scanned image
	[DllImport(recoIO, EntryPoint = "IO_ScanImageWithParameters")]
        internal static extern int IO_ScanImageWithParameters(int SessionHandle, int BitsPerPixel, int Resolution, int Contrast, int Brightness, double Width, double Height,
                                                               int ADF, int Duplex);

	//  Goal: rotate the image by 90/180/270°
	//  Requires: session handle, DIB handle, angle
	//  Result: n/a      
        [DllImport(recoIO, EntryPoint = "IO_Rotate")]
        internal static extern void IO_Rotate(int SessionHandle, int DIBHandle, int Angle);

	//  Goal: flip the image
	//  Requires: session handle, DIB handle
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_Flip")]
        internal static extern void IO_Flip(int SessionHandle, int DIBHandle);

	//  Goal: mirror the image
	//  Requires: session handle, DIB handle
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_Mirror")]
        internal static extern void IO_Mirror(int SessionHandle, int DIBHandle);

	//  Goal: invert the image colors
	//  Requires: session handle, DIB handle
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_Invert")]
        internal static extern void IO_Invert(int SessionHandle, int DIBHandle);

	//  Goal: export a DIB as DDB
	//  Requires: session handle, DIB Handle, bitmap handle (out), palette handle (out)
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_ExportDDB")]
        internal static extern void IO_ExportDDB(int SessionHandle, int DIBHandle, ref int Bitmap, ref int Palette);

	//  Goal: import a DDB as DIB
	//  Requires: session handle, bitmap handle, palette handle
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_ImportDDB")]
        internal static extern int IO_ImportDDB(int SessionHandle, int Bitmap, int Palette);

	//  Goal: aquire DC allowing to us GDI API
	//  Requires: session handle, bitmap handle,
	//  Result: DC
        [DllImport(recoIO, EntryPoint = "IO_AcquireDC")]
        internal static extern int IO_AcquireDC(int SessionHandle, int DIBHandle);

	//  Goal: release DC
	//  Requires: session handle, bitmap handle, DC
	//  Result: n/a
        [DllImport(recoIO, EntryPoint = "IO_ReleaseDC")]
        internal static extern int IO_ReleaseDC(int SessionHandle, int DIBHandle);

	//  Goal: show scanning dialog allowing user to set parameters and scan
	//  Requires: session handle, window handle
	//  Result: n/a
	[DllImport(recoIO, EntryPoint = "IO_ScanImageAsync")]
        internal static extern int IO_ScanImageAsync(int SessionHandle, int WinHandle);

	//  Goal: start scanning without show scanning dialog
	//  Requires: session handle,bit per pixel,resolution,contrast,brightness,width,height, adf,duplex
	//  Result: n/a
	[DllImport(recoIO, EntryPoint = "IO_ScanImageAsyncWithParameters")]
        internal static extern int IO_ScanImageAsyncWithParameters(int SessionHandle, int BitsPerPixel,int Resolution,int Contrast,int Brightness, double Width,double Height,int ADF, int Duplex);

	//  Goal: Get the handle of the next scanned image
	//  Requires: session handle
	//  Result: The DIB handle of the next scanned image
	[DllImport(recoIO, EntryPoint = "IO_GetNextAsyncScannedImage")]
        internal static extern int IO_GetNextAsyncScannedImage(int SessionHandle);

	//  Goal: load a TIF image from a buffer
	//  Requires: session handle, buffer,buffer size, page number
	//  Result: DIB handle
	[DllImport(recoIO, EntryPoint = "IO_LoadTIFImageFromBuffer")]
        internal static extern int IO_LoadTIFImageFromBuffer(int SessionHandle, IntPtr Buffer, int BufferSize,int ImageIndex);

	//  Goal: load a BMP image from a buffer
	//  Requires: session handle, buffer,buffer size
	//  Result: DIB handle
	[DllImport(recoIO, EntryPoint = "IO_LoadBMPImageFromBuffer")]
        internal static extern int IO_LoadBMPImageFromBuffer(int SessionHandle, IntPtr Buffer, int BufferSize);

	//  Goal: load a JPG image from a buffer
	//  Requires: session handle, buffer,buffer size
	//  Result: DIB handle
	[DllImport(recoIO, EntryPoint = "IO_LoadJPGImageFromBuffer")]
        internal static extern int IO_LoadJPGImageFromBuffer(int SessionHandle,IntPtr Buffer, int BufferSize);

	//  Goal: load a PDF image from a buffer
	//  Requires: session handle, buffer,buffer size, page number, DPI Rendering
	//  Result: DIB handle
	[DllImport(recoIO, EntryPoint = "IO_LoadPDFImageFromBuffer")]
        internal static extern int IO_LoadPDFImageFromBuffer(int SessionHandle, IntPtr Buffer, int BufferSize, int ImageIndex, int DPIRendering);

	//  Goal: retrieve the number of pages stored in a TIF file
	//  Requires: session handle, buffer and buffer size
	//  Result: number of pages found
	[DllImport(recoIO, EntryPoint = "IO_CountTIFImagesFromBuffer")]
        internal static extern int IO_CountTIFImagesFromBuffer(int SessionHandle, IntPtr Buffer, int BufferSize);

	//  Goal: retrieve the number of pages stored in a PDF file
	//  Requires: session handle, buffer and buffer size
	//  Result: number of pages found
	//[DllImport(recoIO, EntryPoint = "IO_CountPDFImagesFromBuffer")]
    //    internal static extern int IO_CountPDFImagesFromBuffer(int SessionHandle, IntPtr Buffer, int BufferSize);


	//  Goal: Get the color of a specific pixel
	//  Requires: session handle, image handle, X and Y coordinates
	//  Result: pixel color
	[DllImport(recoIO, EntryPoint = "IO_GetPixel")]
        internal static extern int IO_GetPixel(int SessionHandle, int DIBHandle, int X, int Y);

	//  Goal: Set the color of a specific pixel
	//  Requires: session handle, image handle, X and Y coordinates, pixel color
	//  Result: n/a
	[DllImport(recoIO, EntryPoint = "IO_SetPixel")]
        internal static extern void IO_SetPixel(int SessionHandle, int DIBHandle, int X, int Y, int C);

	//  Goal: Debug the buffer
	//  Requires: session handle, buffer, buffer size
	//  Result: n/a
	[DllImport(recoIO, EntryPoint = "IO_DebugBuffer")]
        internal static extern int IO_DebugBuffer(int SessionHandle, IntPtr Buffer, int BufferSize);

	//  Goal: Set a parameter
	//  Requires: session handle, parameter index, parameter value
	//  Result: n/a
	[DllImport(recoIO, EntryPoint = "IO_SetParameter")]
        internal static extern void IO_SetParameter(int SessionHandle, int ParameterIndex,int ParameterValue);

    //  Goal: start scanning using .tsp file parameters  of Recogniform PaperCapture
    //  Requires: session handle, the .tsp file
    //  Result: n/a
    [DllImport(recoIO, EntryPoint = "IO_ScanImageTSPFile")]
        internal static extern int IO_ScanImageTSPFile(int SessionHandle,string TSPFile);

    //  Goal: start scanning using string of parameters
    //  Requires: session handle, the string of parameters
    //  Result: n/a
    [DllImport(recoIO, EntryPoint = "IO_ScanImageTSPRaw")]
        internal static extern int IO_ScanImageTSPRaw(int SessionHandle, string TSPRaw);
   
    //  Goal: start asynchronous scanning using .tsp file parameters of Recogniform PaperCapture
    //  Requires: session handle, the .tsp file
    //  Result: n/a
    [DllImport(recoIO, EntryPoint = "IO_ScanImageAsyncTSPFile")]
        internal static extern int IO_ScanImageAsyncTSPFile(int Session,string TSPFile);

    //  Goal: start asynchronous scanning using string of parameters
    //  Requires: session handle, the string of parameters
    //  Result: n/a
    [DllImport(recoIO, EntryPoint = "IO_ScanImageAsyncTSPRaw")]
        internal static extern int IO_ScanImageAsyncTSPRaw(int Session, string TSPRow); 

    //  Goal: Make searchable PDF
	//  Requires: session handle, Tif file, Xml file, PDF file, info
	//  Result: n/a
    [DllImport(recoIO, EntryPoint = "IO_MakeSearchablePDF")]
        internal static extern void IO_MakeSearchablePDF(int Session,string TifFileIn,string XMLFileIn,string PDFFileOut,string PDFInfo);
        
    //  Goal: Get scanner status
    //  Requires: session handle
    //  Result: scanner status:
    //          1 Pre-Session
    //          2 Source Manager Loaded
    //          3 Source Manager Opened
    //          4 Source Opened
    //          5 Source Enabled
    //          6 Transfer Ready
    //          7 Transferring
    [DllImport(recoIO, EntryPoint = "IO_GetScannerStatus")]
        internal static extern int IO_GetScannerStatus(int Session);  
        
        
        
    //  Goal: Get scanner name
    //  Requires: session handle, buffer
    //  Result: buffer length
    [DllImport(recoIO, EntryPoint = "IO_GetScannerName")]
    internal static extern int IO_GetScannerName(int Session, StringBuilder ScannerName);

        
    }
}
